import jsonfield
from django.db import models
from django.contrib.sites.models import Site
from django.contrib.auth.models import User
from django.utils.translation import ugettext as _
from cilantro.managers import SiteConfigurationManager


class SiteConfiguration(models.Model):
    title = models.CharField(max_length=50, blank=True,
        help_text=_('If left blank the name will be derived from the site or the hostname'))
    subtitle = models.CharField(max_length=100, blank=True)
    label = models.CharField('status label', max_length=20, blank=True)
    site_logo = models.FileField(upload_to='cilantro/sites', null=True, blank=True)
    site = models.OneToOneField(Site, related_name='configuration+', null=True, blank=True,
        help_text=_('Leave empty to be the default configuration for all sites'))
    footer_content = models.TextField(help_text='Markdown enabled', blank=True)
    google_analytics = models.CharField(max_length=20, null=True, blank=True,
        help_text=_('Just the UA-XXXXXX-XXX code'))
    auth_required = models.BooleanField(default=False)

    objects = SiteConfigurationManager()

    def __unicode__(self):
        text = u'"{}"'.format(self.title or self.site)
        if self.label:
            text = '{} ({})'.format(text, self.label)
        return text

    @property
    def site_name(self):
        return self.title or self.site.name

    @property
    def site_domain(self):
        return self.site.domain


class UserPreferences(models.Model):
    json = jsonfield.JSONField(default=dict)
    user = models.ForeignKey(User, null=True, blank=True)
    session_key = models.CharField(max_length=40, null=True, blank=True)

    class Meta(object):
        verbose_name_plural = 'user preferences'

    def __unicode__(self):
        if self.user_id:
            return u'{}\'s Preferences'.format(self.user)
        return u'Preferences for Session [{}]'.format(self.session_key)



import json
from django.http import HttpResponse
from django.shortcuts import render
from preserialize.serialize import serialize
from cilantro.models import UserPreferences


template = {
    'fields': [':pk', 'json']
}


def app(request):
    return render(request, 'cilantro/index.html')


def preferences(request):
    # Get a user's preferences
    if hasattr(request, 'user') and request.user.is_authenticated():
        kwargs = {'user': request.user}
    # Get this session's preferences
    elif request.session.session_key:
        kwargs = {'session_key': request.session.session_key}
    # Bots..
    else:
        return HttpResponse(status=204)

    # Creating the preferences are transparent, create them if they
    # don't already exist for this user or session.
    preferences, created = UserPreferences.objects.get_or_create(**kwargs)

    if request.method == 'GET':
        return HttpResponse(json.dumps(serialize(preferences, **template)))

    if request.method == 'PUT':
        data = json.loads(request.body)
        data.pop('id', None)
        preferences.json = data
        preferences.save()
        return HttpResponse(status=204)

    return HttpResponse(status=405)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'SiteConfiguration'
        db.create_table('cilantro_siteconfiguration', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=50, blank=True)),
            ('subtitle', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=20, blank=True)),
            ('site_logo', self.gf('django.db.models.fields.files.FileField')(max_length=100, null=True, blank=True)),
            ('site', self.gf('django.db.models.fields.related.OneToOneField')(related_name='configuration', unique=True, blank=True, to=orm['sites.Site'])),
            ('footer_content', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('google_analytics', self.gf('django.db.models.fields.CharField')(max_length=20, null=True, blank=True)),
        ))
        db.send_create_signal('cilantro', ['SiteConfiguration'])

        # Adding model 'UserPreferences'
        db.create_table('cilantro_userpreferences', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('json', self.gf('jsonfield.fields.JSONField')(default={})),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], null=True, blank=True)),
            ('session_key', self.gf('django.db.models.fields.CharField')(max_length=40, null=True, blank=True)),
        ))
        db.send_create_signal('cilantro', ['UserPreferences'])


    def backwards(self, orm):
        # Deleting model 'SiteConfiguration'
        db.delete_table('cilantro_siteconfiguration')

        # Deleting model 'UserPreferences'
        db.delete_table('cilantro_userpreferences')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'cilantro.siteconfiguration': {
            'Meta': {'object_name': 'SiteConfiguration'},
            'footer_content': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'google_analytics': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'configuration'", 'unique': 'True', 'blank': 'True', 'to': "orm['sites.Site']"}),
            'site_logo': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'subtitle': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'})
        },
        'cilantro.userpreferences': {
            'Meta': {'object_name': 'UserPreferences'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'json': ('jsonfield.fields.JSONField', [], {'default': '{}'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['cilantro']


from django.conf.urls import url, patterns, include

urlpatterns = patterns('',
    url(r'', include(patterns('',
        url(r'^api/preferences/$', 'cilantro.views.preferences', name='preferences'),
    ), namespace='cilantro')),
)



from setuptools import setup, find_packages

kwargs = {
    # Packages
    'packages': find_packages(),
    'include_package_data': True,

    # Dependencies
    'install_requires': [
        'serrano>=2.0,<2.1',
    ],

    # Test dependencies
    'tests_require': [
        'coverage',
    ],

    # Optional dependencies
    'extras_require': {},

    # Metadata
    'name': 'cilantro',
    'version': __import__('cilantro').get_version(),
    'author': 'Byron Ruth',
    'author_email': 'b@devel.io',
    'description': 'A client implementation for Serrano 2.x',
    'license': 'BSD',
    'keywords': 'harvest serrano client avocado',
    'url': 'http://cbmi.github.com/cilantro',
    'classifiers': [
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.7',
        'Framework :: Django',
        'Topic :: Internet :: WWW/HTTP',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Healthcare Industry',
        'Intended Audience :: Information Technology',
    ],
}

setup(**kwargs)



import os
from cilantro.models import SiteConfiguration


def cilantro(request):
    from django.conf import settings

    static_url = os.path.join(settings.STATIC_URL, 'cilantro')

    css_url = '{}/stylesheets/css'.format(static_url)
    images_url = '{}/images'.format(static_url)
    javascript_url = '{}/scripts/javascript'.format(static_url)

    # Augment output version (for debugging purposes)
    if settings.DEBUG:
        javascript_url += '/src'
    else:
        javascript_url += '/min'

    try:
        config = SiteConfiguration.objects.get_current(request)
    except SiteConfiguration.DoesNotExist:
        config = None

    return {
        'cilantro': {
            'config': config,
            'static_url': static_url,
            'css_url': css_url,
            'images_url': images_url,
            'javascript_url': javascript_url,
        },
    }
